<?php

namespace App\Service;

use App\Exception;

class ConsumerService {
    
    private $util;
    private $factory;
    private $branchFactory;

    public function __construct (\App\Util\Util $util, 
                                 \App\Factory\ConsumerFactory $factory,
                                 \App\Factory\BranchFactory $branchFactory) {
        $this->util          = $util;
        $this->factory       = $factory;
        $this->branchFactory = $branchFactory;
    }
    
    public function processConsumer ($generalParams, $order) {
        $standartClient = $this->branchFactory->getStandartClient($order["branch"]);

        $consumer = $this->findConsumer($order["customer"], $standartClient);

        if(empty($consumer)) {
            $consumer["customer"]["clientCode"] = $standartClient;
            $consumer["address"] = $order["address"];
            $consumer = $this->registerConsumer($generalParams, $order);
        }
        
        if(empty($consumer["CDMUNICIPIO"])) 
            $consumer["CDMUNICIPIO"] = $this->factory->getCityCode($order["address"]["state"], $order["address"]["city"]);
        
        if(empty($consumer["CDMUNICIPIO"])) 
            $consumer["CDMUNICIPIO"] = $this->branchFactory->getBranchCityCode($order["branch"]);
        
        $consumer["CDBAIRRO"] = $this->factory->getNeighborhoodCode($order["address"], $consumer["CDMUNICIPIO"]);
        $consumer["DSAREAATEND"] = "";
        
        return $consumer;
    }
    
    private function findConsumer($customer, $standartClient) {
        $consumer = [];

        if(!empty($customer["email"])) 

            $consumer = $this->factory->getConsumerByEmail($customer["email"]);

        if (empty($consumer) && !empty($customer["telephone"])) 

            $consumer = $this->factory->getConsumerByTelNumber($customer["telephone"]);
        
        if (empty($consumer) && !empty($customer["document"])) 

            $consumer = $this->factory->getConsumerByTaxPayer($customer["document"]);
        
        if(!empty($consumer) && ($consumer["CDCLIENTE"] != $standartClient["CDCLIENTE"])) 
            $consumer = [];
            
        return $consumer;
    }
    
    public function registerConsumer ($generalParams, $customer) {

        $stdClient = $this->branchFactory->getStandartClient($generalParams["CDFILIAL"]);
        $consumerT = $this->branchFactory->getStandartConsumerType($generalParams["CDFILIAL"]);
        $consumer  = "SDE".$this->util->newCode("CONSUMIDOR".$stdClient["CDCLIENTE"], 18);
        
        $customer["customer"]["email"]     = !empty($customer["customer"]["email"]) ? $customer["customer"]["email"] : null;
        $customer["customer"]["telephone"] = !empty($customer["customer"]["telephone"]) ? $customer["customer"]["telephone"] : null;
        $customer["customer"]["document"]  = !empty($customer["customer"]["document"]) ? $customer["customer"]["document"] : null;

        $consumerDetails["CDCONSUMIDOR"] = $consumer;
        $consumerDetails["CDMUNICIPIO"]  = $this->factory->getCityCode($customer["address"]["state"], $customer["address"]["city"]);

        if (empty($consumerDetails["CDMUNICIPIO"])){
            $consumerDetails["CDMUNICIPIO"] = $this->branchFactory->getBranchCityCode($generalParams["CDFILIAL"]);
            if(empty($consumerDetails["CDMUNICIPIO"])){
                throw new Exception\ParamsException("Não foi encontrado o código do município.");
            }
        }
            
        
        $mergeCandidates = array($stdClient, $consumerT);

        foreach ($mergeCandidates as $candidate) 
            $consumerDetails = array_merge($consumerDetails, $candidate);

        $consumer = $this->factory->registerConsumer(array_merge($consumerDetails, $customer));
        
        return $consumer;
    }
}
